//+------------------------------------------------------------------+
//|                                            FxCraftPipsometr.java |
//|                                         FxCraft :: Jan Jablonski |
//|                                           Copyright 2013 FxCraft |
//|                                 licencja: Creative Commons BY-SA |
//|                                              http://fxcraft.biz/ |
//+------------------------------------------------------------------+
package jforex;
import java.util.*;
import com.dukascopy.api.*;
import java.text.*;
import java.awt.Color;

public class FxCraftPipsometer implements IStrategy {
    private IEngine engine;
    private IConsole console;
    private IHistory history;
    private IContext context;
    private IChart chart;
    
    private String comment;
    private int horizontalPosition = javax.swing.SwingConstants.RIGHT;
    private int verticalPosition = javax.swing.SwingConstants.TOP;
    
    private Color color = Color.RED;
    
    @Configurable("Instrument:")
    public Instrument instrumentThis = Instrument.EURUSD;
       
    public void onStart(IContext context) throws JFException {
        this.engine = context.getEngine();
        this.console = context.getConsole();
        this.history = context.getHistory();
        this.context = context;
        
        this.chart = context.getChart(instrumentThis);
        this.comment = "";
        this.chart.setCommentHorizontalPosition(horizontalPosition);
        this.chart.setCommentVerticalPosition(verticalPosition);
        chart.setCommentColor(color); 
    }

    public void onAccount(IAccount account) throws JFException {
    }

    public void onMessage(IMessage message) throws JFException {
    }

    public void onStop() throws JFException {
        chart.comment("");
    }

    public void onTick(Instrument instrument, ITick tick) throws JFException {
        if(instrument == instrumentThis){
            this.comment = "";
            double lpm_pips=0;
            double lpm_currency=0;
            int lpm_long=0;
            double lpm_long_amount=0;
            int lpm_short=0;
            double lpm_short_amount=0;
            double lpm_pips_tosl=0;
            double lpm_currency_tosl=0;
            double lpm_pips_realized=0;
            double lpm_currency_realized=0;
            
            //lpm_long, lpm_short, lpm_pips, lpm_pips_tosl
            for (IOrder orderr : engine.getOrders(instrumentThis)) {
                    if(orderr.isLong()){
                        lpm_long++;
                        lpm_long_amount += orderr.getAmount();
                    }else{
                        lpm_short++;
                        lpm_short_amount += orderr.getAmount();
                    }
                    lpm_pips += orderr.getProfitLossInPips();
                    lpm_currency += orderr.getProfitLossInAccountCurrency();
                    if(orderr.getStopLossPrice() != 0){
                        double pips_this_order = NormalizeDouble((orderr.getStopLossPrice() - orderr.getOpenPrice())/instrumentThis.getPipValue(), 1);
                        lpm_pips_tosl += pips_this_order;
                        lpm_currency_tosl += NormalizeDouble(pips_this_order * orderr.getAmount() * 100, 2);
                    }
            }
            
            //lpm_pips_realized
            Date d_from = new Date();
            Date d_to = new Date(tick.getTime());
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd");
            String d_tmp = formatter.format(d_to);
            try{
                d_from = formatter.parse(d_tmp);
            }catch(Exception e){
                console.getErr().println("wyjatek przy parsowaniu: "+e);
            }
            synchronized(this){
            List<IOrder> historyOrders = history.getOrdersHistory(instrumentThis, d_from.getTime(), d_to.getTime());
            for(IOrder orderr : historyOrders) {
                  lpm_pips_realized += orderr.getProfitLossInPips();
                  //lpm_currency_realized += orderr.getProfitLossInAccountCurrency();   /nie wiem czemu nie dziala
                  lpm_currency_realized += NormalizeDouble((orderr.getProfitLossInPips() * orderr.getAmount())*instrumentThis.getPipValue(), 2);
            }
            }
            
            this.comment += "Summary long/amount: " + NormalizeDouble(lpm_long, 3) + " / " + NormalizeDouble(lpm_long_amount, 3);
            this.comment += "\nSummary short/amount: " + NormalizeDouble(lpm_short, 3) + " / " + NormalizeDouble(lpm_short_amount, 3);
            this.comment += "\nP/L pips/currency: " + NormalizeDouble(lpm_pips, 1) + " / " + NormalizeDouble(lpm_currency, 2);
            this.comment += "\nSecured pips/currency: " + NormalizeDouble(lpm_pips_tosl, 1) + " / " + NormalizeDouble(lpm_currency_tosl, 2);
            this.comment += "\nRealized pips/currency: " + NormalizeDouble(lpm_pips_realized, 1) + " / " + NormalizeDouble(lpm_currency_realized, 2);
            
            chart.comment(this.comment);
        }
    }
    
    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) throws JFException {
    }
    
    double NormalizeDouble(double operand, int miejcsa) {
       String wzor = "0.0";
       for(int i=1;i<miejcsa;i++){
           if(i==miejcsa-1) wzor += "#";
           else wzor += "0";
       }
       DecimalFormat applydeci = new DecimalFormat(wzor);
       Double result = new Double(applydeci.format(operand).replace(',', '.')).doubleValue();
       return(result);
    }
}